//=========================================================================
//
//	FILE:				CP_Application.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Main application class for a CPLAT application. This
//						class has no built in support for managing documents.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 204 $
//						Latest Checkout:  $Date: 2005-06-06 22:15:38 -0700 (Mon, 06 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Application
#define _H_CP_Application
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Application_Imp_CE.h"			// Application implementation class for MacOS CarbonEvents
#include "CP_Application_Imp_Win.h"			// Application implementation class for Windows
#include "CP_M_CommandHandler.h"
#include "CP_Printer.h"
#include "CP_Window.h"

#if TARGET_OS_LINUX
	#include "CP_Application_Imp_Gtk.h"		// Application implementation class for Linux / Gtk
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Application
*	@brief		Main application class.
*	
*				Main application class for a CPLAT application. This class has no built
*				in support for managing documents.
*
*				The actual implementation of this class is handled by the CP_Application_Imp
*				class which will have an implementation for the specific platform the application
*				is compiled for, such as MacOS Carbon, MacOS WNE, Windows and Linux.
*
*				You will typically override this class in your application, or CP_DocumentApp if
*	 			you want built-in support for documents.
*
*				Then in your main routine you will create an instance of your app object, and call
*				the Run() method to run the event loop, as shown here (note: CP_Startup code removed
*				at beginning of main):
*
*	@code
*				int main()
*				{
*					try
*					{	
*						CP_Document_Application		application;
*						application.Run();	
*					}
*					catch(...)
*					{
*					}
*	
*					return( 0 );
*				}
*	@endcode
*/

template <class DOC_POLICY, class PRINT_POLICY, class UNDO_POLICY>
class CP_EXPORT CP_Application : public CP_Application_Imp
{
public:
	typedef DOC_POLICY					DocPolicyType;
	typedef PRINT_POLICY				PrintPolicyType;
	typedef UNDO_POLICY					UndoPolicyType;

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Application();
	//! Destructor.
	virtual								~CP_Application() throw();
	//@}

	//! @name Policies
	//@{
	//! Return the document, undo and print policy.
			DocPolicyType&				GetDocumentPolicy()								{ return( fDocumentPolicy ); }
			PrintPolicyType& 			GetPrintPolicy()								{ return( fPrintPolicy ); }
			UndoPolicyType& 			GetUndoPolicy()									{ return( fUndoPolicy ); }
	//@}

protected:			
	//! @name AboutBox
	//@{
	virtual	CP_String					GetAboutBoxDescription() const					{ return( CP_String("") ); }
	//@}

	//! @name Commands
	//@{
	virtual	void						HandleOpenApplication();
	virtual	bool						HandleQuitApplication();

											// File menu commands
	virtual	void						HandleNewUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutWasHandled,
												CP_CommandNo& inCommandNo );
	virtual	void						HandleNewCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandleOpenUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutWasHandled,
												CP_CommandNo& inCommandNo );
	virtual	void						HandleOpenCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandlePageSetupUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutWasHandled,
												CP_CommandNo& inCommandNo );
	virtual	void						HandlePageSetupCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	virtual	void						HandlePrintUpdateCommandStatusCommand(
												CP_CommandStatus& inOutCommandStatus,
												bool& inOutWasHandled,
												CP_CommandNo& inCommandNo );
	virtual	void						HandlePrintCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );

	virtual	void						HandleAboutBoxCommand(
												OSStatus& inOutStatus,
												CP_CommandNo& inCommandNo );
	//@}

	//! @name Documents
	//@{
	virtual SInt32						CountDocuments();
	virtual	CP_ScriptableObject_Imp		*GetNthDocument( SInt32 inPosition );
	//@}

	//! @name Printing
	//@{
	virtual	void						HandlePrinting( CP_Printer */*inPrinter*/ )		{}
	//@}

	// class data
protected:
	DOC_POLICY							fDocumentPolicy;
	PRINT_POLICY						fPrintPolicy;
	UNDO_POLICY							fUndoPolicy;
	
	CP_PrintSignal						fPrintSignal;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name AppleEvents
	//@{
	virtual	void						HandleOpenDocumentAppleEvent(
												const AppleEvent& inEvent,
												AppleEvent& inOutReply,
												SInt32 inRefCon );

	virtual	CP_ScriptableObject_Imp 	*DoAE_CreateElement(
												DescType inDesiredClass,
												DescType inInsertLocation,
												const AppleEvent& inEvent,
												AppleEvent& inOutAEReply );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Application

