//=========================================================================
//
//	FILE:				CP_Event.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class for handling events in CPLAT. There are specific
//						implementor classes that provide the actual work of
//						this class depending on the platform.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 204 $
//						Latest Checkout:  $Date: 2005-06-06 22:15:38 -0700 (Mon, 06 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Event
#define _H_CP_Event
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Event_Imp_CE.h"
#include "CP_Event_Imp_Win.h"
#include "CP_OSStatus.h"
#include "CP_SignalSlot.h"
#include "CP_Types.h"

#if TARGET_OS_LINUX
	#include "CP_Event_Imp_Gtk.h"
#endif

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_Event_Base;

typedef CP_SigSlot::CP_Signal1<CP_Event_Base&>									CP_EventSignal;

/*!
*	@class		CP_Event
*	@brief		Class that represents an event in a CPLAT application.
*	
*				There are specific implementor classes that provide the actual work
*				of this class depending on the platform.
*/
class CP_EXPORT CP_Event_Base : public CP_Event_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event_Base();
										CP_Event_Base( const CP_Event_Base& inEventBase );
										CP_Event_Base(
												UInt32 inClass, 
												UInt32 inKind );
	//! Destructor.
	virtual								~CP_Event_Base();
	//@}
			
	//! @name Status
	//@{
										/*!
										*	Returns the status of the event. This will be the status
										*	returned to the underlying OS.
										*	
										*	@exception	none
										*	
										*	@return		OSStatus		The OSStatus code returned to the OS.
										*/
			OSStatus					GetStatus()										{ return( fStatus ); }

										/*!
										*	Sets the status of the event. This will be the status
										*	returned to the underlying OS.
										*
										*	@param		inStatus		The status thats going to be returned to
										*								the underlying OS.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetStatus( OSStatus inStatus )					{ fStatus = inStatus; }
	//@}

protected:

	// class data
protected:
	OSStatus							fStatus;		/*! Current clipping region. */
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	#if TARGET_API_MAC_CARBON
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event_Base(
												EventRef inEvent,
												EventHandlerCallRef inHandlerCallRef );
	//@}
	#endif	// TARGET_API_MAC_CARBON
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event_Base(
												HWND inWindow,
												WPARAM inWParam, 
												LPARAM inLParam );
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event_Base( GdkEvent *inEvent );
	//@}
#endif
};

template <UInt32 CLASS, UInt32 KIND>
class CP_Event : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event();
										CP_Event( const CP_Event_Base& inBaseEvent );
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	#if TARGET_API_MAC_CARBON
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event(
												EventRef inEvent,
												EventHandlerCallRef inHandlerCallRef );
	//@}
	#endif	// TARGET_API_MAC_CARBON
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event(
												HWND inWindow,
												WPARAM inWParam, 
												LPARAM inLParam );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

template <UInt32 CLASS, UInt32 KIND>
CP_Event<CLASS, KIND>::CP_Event()
	: CP_Event_Base(CLASS, KIND)
{
}

template <UInt32 CLASS, UInt32 KIND>
CP_Event<CLASS, KIND>::CP_Event( const CP_Event_Base& inBaseEvent )
	: CP_Event_Base(inBaseEvent)
{
}

#if TARGET_API_MAC_CARBON

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

template <UInt32 CLASS, UInt32 KIND>
CP_Event<CLASS, KIND>::CP_Event( EventRef inEvent, EventHandlerCallRef inHandlerCallRef)
	: CP_Event_Base(inEvent, inHandlerCallRef)
{
}

#endif	// TARGET_API_MAC_CARBON

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Windows Specific ===
#pragma mark -
#endif

template <UInt32 CLASS, UInt32 KIND>
CP_Event<CLASS, KIND>::CP_Event( HWND inWindow, WPARAM inWParam, LPARAM inLParam )
	: CP_Event_Base( inWindow, inWParam, inLParam )
{
}

#endif	// TARGET_API_MAC_CARBON

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Event

