//=========================================================================
//
//	FILE:				CP_Thread.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class to provide thread support for an object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 225 $
//						Latest Checkout:  $Date: 2005-07-01 17:21:21 -0700 (Fri, 01 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Thread
#define _H_CP_Thread
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"

#if CP_ThreadingAPI_PThread
	#include <pthread.h>
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_M_Thread
*	@brief	Mix-in class for using threads in CPLAT. For each object that needs to run in
*			a separate thread, mix-in this class and override the Run() method, which is
*			the routine that does the actual work.
*/
class CP_EXPORT CP_M_Thread
{
public:
	enum
	{
		#if CP_ThreadingAPI_WinThread
			kPriorityIdle = THREAD_PRIORITY_IDLE,
			kPriorityLowest = THREAD_PRIORITY_LOWEST,
			kPriorityBelowNormal = THREAD_PRIORITY_BELOW_NORMAL,
			kPriorityNormal = THREAD_PRIORITY_NORMAL,
			kPriorityHighest = THREAD_PRIORITY_HIGHEST,
			kPriorityCritical = THREAD_PRIORITY_TIME_CRITICAL,
			kPriorityError = THREAD_PRIORITY_ERROR_RETURN
		#endif
		#if CP_ThreadingAPI_Cooperative
			kPriorityIdle = 0,
			kPriorityLowest = 0,
			kPriorityBelowNormal = 0,
			kPriorityNormal = 0,
			kPriorityHighest = 0,
			kPriorityCritical = 0,
			kPriorityError = LONG_MAX
		#endif
		#if CP_ThreadingAPI_MPTask
			kPriorityIdle = 0,
			kPriorityLowest = 1,
			kPriorityBelowNormal = 10,
			kPriorityNormal = 100,
			kPriorityHighest = 10000,
			kPriorityCritical = 5000,
			kPriorityError = LONG_MAX
		#endif
		#if CP_ThreadingAPI_PThread
			kPriorityIdle = 0,
			kPriorityLowest = 1,
			kPriorityBelowNormal = 10,
			kPriorityNormal = 50,
			kPriorityHighest = 100,
			kPriorityCritical = 90,
			kPriorityError = LONG_MAX
		#endif
	};

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_Thread();
	//! Destructor.
	virtual								~CP_M_Thread();
	//@}

	//! @name Control
	//@{
	static	void						YieldThread( CP_M_Thread *inThread = NULL );
	static	void						SleepThread( UInt32 numMilliSecs );

			SInt32						GetPriority();
			SInt32						SetPriority(SInt32 newVal);
			void						Suspend();
			void						Finish();
			void						Resume();
			bool						IsSuspend();

										/*!
										*	Return the status of the has completed flag to indicate if this
										*	thread has completed. 
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the thread has completed.
										*/
			bool						HasCompleted()									{ return( fHasCompleted ); }

			void						Kill();
	//@}

	//! @name Setup
	//@{
			void						SetupThread();
	//@}

protected:
	//! @name Running
	//@{
										/*!
										*	Subclasses must override this to provide the routine that does the
										*	work in the tread. 
										*	
										*	@exception	none
										*	
										*	@return		UInt32		Result of the thread.
										*/
	virtual UInt32						Run() = 0;
	//@}

	// class data
private:
	bool								fHasCompleted;
	bool								*fDestructed;

protected:
	#if CP_ThreadingAPI_Cooperative
		ThreadID						fThreadID;				/*! A MacOS ThreadID for this thread. */
	#endif
	
	#if CP_ThreadingAPI_MPTask
		MPQueueID						fQueueID;
		MPTaskID						fTaskID;
		SInt32							fTaskWeight;
	#endif

	#if CP_ThreadingAPI_PThread
		bool							fThreadIDValid;
		pthread_t						fThreadID;
		pthread_cond_t					fRefresh;
		pthread_mutex_t					fMutex;
	#endif

	#if CP_ThreadingAPI_WinThread
		HANDLE							fThreadHandle;		/*! Windows thread HANDLE for this thread. */
		unsigned						fThreadID;			/*! Thread id for the thread. */
		bool							fThreadSuspend;		/*! Flag to indicate if the thread is suspended. */
	#endif

//========================================================================
// Platform specific routines
//========================================================================
protected:	
	//! @name Getters
	//@{
	#if CP_ThreadingAPI_Cooperative
										/*!
										*	Return the thread id, MacOS version. 
										*	
										*	@exception	none
										*	
										*	@return		ThreadID	MacOS id of the thread.
										*/
			ThreadID					GetID()									{ return( fThreadID ); }
	#endif

	#if CP_ThreadingAPI_WinThread
										/*!
										*	Return the thread id, Windows version. 
										*	
										*	@exception	none
										*	
										*	@return		ThreadID	MacOS id of the thread.
										*/
			DWORD						GetID()									{ return( fThreadID ); }
	#endif
	//@}

private:
	//! @name Running
	//@{
	#if CP_ThreadingAPI_Cooperative
		static pascal void				*ThreadEntry( void *inParameter );
	#endif
	
	#if CP_ThreadingAPI_MPTask
		static pascal ::OSStatus		ThreadEntry( void *inParameter );
	#endif
		
	#if CP_ThreadingAPI_PThread
		static void						*ThreadEntry( void *inThread );
	#endif

	#if CP_ThreadingAPI_WinThread
		static unsigned WINAPI			ThreadEntry( LPVOID inParameter );
	#endif
	//@}
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Thread

