//=========================================================================
//
//	FILE:				CP_MailMessage.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to encompass an internet mail type message. This class
//						is used by the CP_POP3 and CP_SMTP protocols to send and
//						receive messages.
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_MailMessage
#define _H_CP_MailMessage
#ifndef TARGET_OS_LINUX
	#pragma once
#endif


	// CPLAT_Framework
#include "CP_DataEncoder.h"
#include "CP_FileSystemSpec.h"
#include "CP_InternetDefines.h"
#include "CP_InternetMessage.h"
#include "CP_STLArrayDefines.h"
#include "CP_String.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// forward declaration
class CP_EXPORT CP_MailMessage;
class CP_EXPORT CP_MailAttachment;		// attachment to a mail message

#if ! TARGET_OS_LINUX
	typedef std::vector<CP_MailMessage *, std::allocator<CP_MailMessage *> >							CP_MailMessage_Array;
	typedef std::vector<CP_MailMessage *, std::allocator<CP_MailMessage *> >::iterator					CP_MailMessage_Iterator;
	typedef std::vector<CP_MailMessage *, std::allocator<CP_MailMessage *> >::reverse_iterator			CP_MailMessage_ReverseIterator;

	typedef std::vector<CP_MailAttachment *, std::allocator<CP_MailAttachment *> >						CP_MailAttachment_Array;
	typedef std::vector<CP_MailAttachment *, std::allocator<CP_MailAttachment *> >::iterator			CP_MailAttachment_Iterator;
	typedef std::vector<CP_MailAttachment *, std::allocator<CP_MailAttachment *> >::reverse_iterator	CP_MailAttachment_ReverseIterator;
#else
	typedef std::vector<CP_MailMessage * >																CP_MailMessage_Array;
	typedef std::vector<CP_MailMessage * >::iterator													CP_MailMessage_Iterator;
	typedef std::vector<CP_MailMessage * >::reverse_iterator											CP_MailMessage_ReverseIterator;

	typedef std::vector<CP_MailAttachment * >															CP_MailAttachment_Array;
	typedef std::vector<CP_MailAttachment * >::iterator													CP_MailAttachment_Iterator;
	typedef std::vector<CP_MailAttachment * >::reverse_iterator											CP_MailAttachment_ReverseIterator;
#endif

#if TARGET_CC_METROWERKS
#pragma mark === CP_MailMessage ==
#endif

/*!
*	@class		CP_MailMessage
*	@brief		A CP_InternetMessage class used to manage messages sent and received by POP3 and SMTP protocols.
*
*/
class CP_EXPORT CP_MailMessage : public CP_InternetMessage
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MailMessage();
	//! Destructor.
										~CP_MailMessage() throw();
	//@}

	//! @name Attachments
	//@{
				bool					HaveAttachments();
				void					AddAttachment( CP_MailAttachment *inAttachment );
				void					RemoveAttachment( CP_MailAttachment *inAttachment );
				void					RemoveAttachmentAt( UInt16 inItemIndex );
				void					RemoveAllAttachments();
				UInt32					GetNumberAttachments();
				
										/*!
										*	Return the list of attachments for this message.
										*	
										*	@exception	none
										*
										*	@return		CP_MailAttachment_Array		List of attachments.
										*/
				CP_MailAttachment_Array	*GetAttachmentsList()							{ return( fAttachments ); }
				CP_MailAttachment		*GetNthAttachment( UInt32 inItemIndex );
	//@}
				
	//! @name Getters
	//@{
				CP_String				GetHeaderItem( const CP_String& inName );

										/*!
										*	Return the To field for this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		Text of the to field.
										*/
				CP_String				GetTo() const									{ return( fTo ); }

										/*!
										*	Return the Subject field for this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		Text of the subject field.
										*/
				CP_String				GetSubject() const								{ return( fSubject ); }

										/*!
										*	Return the From field for this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		Text of the from field.
										*/
				CP_String				GetFrom() const									{ return( fFrom ); }

										/*!
										*	Return the Date field for this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		Text of the date field.
										*/
				CP_String				GetDate() const									{ return( fDate ); }

				SInt32					GetGMTOffset();

										/*!
										*	Return the list of people this message is being sent to.
										*	
										*	@exception	none
										*
										*	@return		CP_Strings_Array	List of people the message is sent to.
										*/
				CP_Strings_Array		*GetSendToList()								{ return( fSendToList ); }

										/*!
										*	Return the list of people this message is being sent ccd to.
										*	
										*	@exception	none
										*
										*	@return		CP_Strings_Array	List of people the message is ccd to.
										*/
				CP_Strings_Array		*GetCCList()									{ return( fCCList ); }

										/*!
										*	Return the list of people this message is being bccd to.
										*	
										*	@exception	none
										*
										*	@return		CP_Strings_Array	List of people the message is bccd to.
										*/
				CP_Strings_Array		*GetBccList()									{ return( fBCCList ); }

										/*!
										*	Return a string that describes the version of the mailer program sending the message.
										*	
										*	@exception	none
										*
										*	@return		CP_String			XMailer version string.
										*/
				CP_String				GetXMailerVers()								{ return( fXMailerVers ); }
	//@}

	//! @name Header
	//@{
	virtual		void					FormatHeader();
	//@}

	//! @name MIME - Getters
	//@{
										/*!
										*	Return the MIME version of this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String			MIME version of message.
										*/
	virtual		CP_String				GetMIMEVers()										{ return( fMIMEVers ); }

										/*!
										*	Return the MIME content type of this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String			MIME content type of message.
										*/
	virtual		CP_String				GetContentType()									{ return( fContentType ); }

										/*!
										*	Return the boundry type between MIME sections of this message.
										*	
										*	@exception	none
										*
										*	@return		CP_String			Boundry type between MIME sections.
										*/
	virtual		CP_String				GetBoundary()										{ return( fBoundary ); }
				CP_String				GetBoundaryHeader();
				CP_String				GetBoundaryTerminator();
	//@}

	//! @name MIME - Setters
	//@{
										/*!
										*	Sets the MIME version of this message.
										*
										*	@param		inMIMEVers		The MIME version of this message.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetMIMEVers( const CP_String& inMIMEVers )			{ fMIMEVers = inMIMEVers; }

										/*!
										*	Sets the content type of this message.
										*
										*	@param		inContentType		The content type of this message.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetContentType( const CP_String& inContentType )	{ fContentType = inContentType; }

										/*!
										*	Sets the boundry marker of this message.
										*
										*	@param		inBoundary		The boundary marker of this message.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetBoundary( const CP_String& inBoundary )			{ fBoundary = inBoundary; }
	//@}

	//! @name Recipients
	//@{
				bool					HaveToList();
				bool					HaveCCRecipients();
				bool					HaveBccRecipients();

				void					AddToRecipient( const CP_String& inRecipient );
				void					AddCCRecipient( const CP_String& inRecipient );
				void					AddBCCRecipient( const CP_String& inRecipient );
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the To field of this message.
										*
										*	@param		inTo		Text of the To field.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetTo( const CP_String& inTo )					{ fTo = inTo; }

										/*!
										*	Sets the Subject field of this message.
										*
										*	@param		inSubject	Text of the Subject field.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetSubject( const CP_String& inSubject )		{ fSubject = inSubject; }

										/*!
										*	Sets the From field of this message.
										*
										*	@param		inFrom		Text of the From field.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetFrom( const CP_String& inFrom )				{ fFrom = inFrom; }

										/*!
										*	Sets the Date field of this message.
										*
										*	@param		inTo		Text of the date field.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetDate( const CP_String& inDate )				{ fDate = inDate; }

										/*!
										*	Sets the XMailerVers field of this message.
										*
										*	@param		inTo		Text of the XMailerVers field.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
				void					SetXMailerVers( const CP_String& inMailerVers )	{ fXMailerVers = inMailerVers; }
	//@}

protected:
	//! @name Date
	//@{
	virtual		CP_String				FormatDate();
															
				CP_String				GetToRecipientsForHeader();
				CP_String				GetCCRecipientsForHeader();
	//@}

private:
	//! @name Initialization
	//@{
				void					Init();
	//@}
										
	// Data members
protected:
	CP_String						fTo;				/*! Who the message is being sent to. */
	CP_String						fSubject;			/*! Subject of the message. */
	CP_String						fFrom;				/*! Who the message is from. */
	CP_String						fDate;				/*! Date & time message was sent */
	CP_String						fReplyTo;			/*! Reply to person. */
	CP_String						fXMailerVers;		/*! Defines which mailer sent the message. */
	
	CP_Strings_Array				*fSendToList;		/*! List of people to send message to. */
	CP_Strings_Array				*fCCList;			/*! List of people to cc message to. */
	CP_Strings_Array				*fBCCList;			/*! List of people to bcc message to. */

		// attachments
	CP_MailAttachment_Array			*fAttachments;		/*! List of attachments for this message. */
	CP_String						fContentType;		/*! Content type of message. */
	CP_String						fBoundary;			/*! Boundary between elements in message. */
	CP_String						fMIMEVers;			/*! MIME version used in message. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_MailMessage ==
#endif

/*!
*	@class		CP_MailMessageAddress
*	@brief		A helper class to handle formatting of e-mail address for use in CP_MailMessages.
*
*/
class CP_EXPORT CP_MailMessageAddress
{
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MailMessageAddress();
										CP_MailMessageAddress( const CP_MailMessageAddress& inAddress );
										CP_MailMessageAddress( const CP_String& inAAddress );
										CP_MailMessageAddress(
												const CP_String& inName,
												const CP_String& inAddress );
	//@}

	//! @name Getters
	//@{
			CP_String					GetFormatForSMTPMessage() const;

										/*!
										*	Returns the email address for the message.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		The email address.
										*/
			CP_String					GetEMailAddress() const										{ return( fEmailAddress ); }

										/*!
										*	Returns the friendly name of the address, if any.
										*	
										*	@exception	none
										*
										*	@return		CP_String const		Friendly version of the name, if any.
										*/
			CP_String					GetName() const												{ return( fName ); }
	//@}

	//! @name operators
	//@{
			CP_MailMessageAddress&		operator = ( const CP_MailMessageAddress& inAddress );
	//@}

	// Data members
protected:
	CP_String        				fName;				/*! Name portion of the address. Example: John Doe */
	CP_String       				fEmailAddress;		/*! Email portion of the address. Example: JohnDoe@address.com. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_MailAttachment ==
#endif

/*!
*	@class		CP_MailAttachment
*	@brief		A helper class to handle e-mail attachments for use in CP_MailMessages.
*
*/
class CP_EXPORT CP_MailAttachment
{
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MailAttachment();
										CP_MailAttachment( const CP_MailAttachment& inAttachment );
	//! Destructor.
	virtual								~CP_MailAttachment();
	//@}
	
	//! @name operators
	//@{
			CP_MailAttachment&			operator = ( const CP_MailAttachment& inAttachment );

			void						Attach( const CP_FileSystemSpec& inAttachmentSpec );
	//@}
			
	//! @name Getters
	//@{
			CP_String					GetHeader();

										/*!
										*	Returns the title of this attachment.
										*	
										*	@exception	none
										*
										*	@return		CP_String			Title of the attachment.
										*/
			CP_String					GetTitle()										{ return( fTitle ); }

										/*!
										*	Returns the file name of this attachment.
										*	
										*	@exception	none
										*
										*	@return		CP_String			File name of the attachment.
										*/
			CP_String					GetFileName()									{ return( fAttachmentSpec.GetName() ); }

										/*!
										*	Returns the data of this attachment.
										*	
										*	@exception	none
										*
										*	@return		cnst CP_HandleBlock *	Data of the attachment.
										*/
			const CP_HandleBlock*		GetData() const										{ return( fData ); }
			UInt32						GetDataLength();

										/*!
										*	Returns the MIME content of this attachment.
										*	
										*	@exception	none
										*
										*	@return		CP_String		MIME content of the attachment.
										*/
			CP_String					GetContentType()								{ return( fContentType ); }
			CP_String					GetContentEncoding();
			
										/*!
										*	Returns the CP_DataEncoder used to encode this attachment.
										*	
										*	@exception	none
										*
										*	@return		CP_DataEncoder		Data encoder used to encode the attachment.
										*/
			CP_DataEncoder				*GetEncoder()									{ return( fEncoder ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the title for this attachment.
										*
										*	@param		inTitle		The title for the attachment.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetTitle( const CP_String& inTitle )			{ fTitle = inTitle; }

										/*!
										*	Sets the data encoder for this attachment.
										*
										*	@param		inEncoder		The data encoder for the attachment.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetEncoder( CP_DataEncoder *inEncoder )			{ fEncoder = inEncoder; }

										/*!
										*	Sets the data for this attachment.
										*
										*	@param		inData					The data for the attachment.
										*	@param		inTransferOwnership		Pass \a true to transfer ownership if the CP_DataHandleBlock, 
										*										or \a false if this object should leave the handle block alone.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetData ( CP_HandleBlock *inData, bool inTransferOwnership=false );

										/*!
										*	Sets the content type for this attachment.
										*
										*	@param		inContentType		The content type for the attachment.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetContentType ( CP_String inContentType );

										/*!
										*	Sets the content for this attachment from the passed in string.
										*
										*	@param		inContents		The contents for the attachment.
										*	@param		inContentType	The type of content.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetStringContents(
												const CP_String& inContents,
												CP_String inContentType = kCP_Header_DefaultContentType );
	//@}

protected:

private:
	//! @name Initialization
	//@{
			void						Init();
	//@}

	// Data members
protected:
	CP_String						fTitle;					/*!< Name (title) of the attachment. */
	CP_FileSystemSpec				fAttachmentSpec;		/*!< CP_FileSystemSpec for the file containing the attachment data. */
	CP_String						fContentType;
	CP_DataEncoder					*fEncoder;				/*!< CP_DataEncoder used to encode the data for this object. */
	CP_HandleBlock					*fData;					/*!< CP_HandleBlock object containing the data of this attachment. */
	bool							fDataOwned;				/*!< Are we responsible for freeing the fData object? */ 
		// MIME
	CP_String						fMIMEVers;				/*!< MIME version of the attachment. */
	CP_String						fBoundary;				/*!< Boundary marker for the attachment. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MailMessage


