//=========================================================================
//
//	FILE:				CP_ObjectDescriptions.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage a list of CP_ObjectDescriptions that
//						are used to create objects in a CP_M_Persistant state.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_ObjectDescriptions
#define _H_CP_ObjectDescriptions
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueList.h"
#include "CP_M_Persist.h"
#include "CP_NoCopy.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

#if TARGET_CC_METROWERKS
#pragma mark === CP_ObjectDescription ==
#endif

/*!
*	@class		CP_ObjectDescription
*	@brief		A class that holds a CP_M_Persist object and a CP_DataValueList that contains
*				one or more objects that hold a name and data value used in the initialization
*				of the stored CP_M_Persist object.
*/
class CP_EXPORT CP_ObjectDescription
{
public:
											// Construction / destruction
										CP_ObjectDescription();
										CP_ObjectDescription(
												CP_M_Persist *inPersistObject,
												CP_DataValueList& inDataValues );
										CP_ObjectDescription( const CP_ObjectDescription& inObjectDescription );
	virtual								~CP_ObjectDescription();

											// Getters
										/*!
										*	Returns the CP_M_Persist object that was created.
										*
										*	@return	CP_M_Persist *			Returns the CP_M_Persist object.
										*/
			CP_M_Persist				*GetObject()												{ return( fPersistObject ); }

										/*!
										*	Returns a CP_DataValueList that contains the data needed to finish
										*	initializing the CP_M_Persist object.
										*
										*	@return	CP_DataValueList		List of names/data values pairs.
										*/
			CP_DataValueList			GetDataValues()												{ return( fDataValues ); }

											// Operators
	CP_ObjectDescription&				operator = ( const CP_ObjectDescription& inObjectDescription );

											// Testers
										/*!
										*	Returns true if this object holds a constructed CP_M_Persist object.
										*   This does not mean the object has been initialized yet, just created.
										*
										*	@return	bool					Returns true if the CP_M_Persist object has been created.
										*/
			bool						IsValid()													{ return( fPersistObject != NULL ); }

protected:

	// class data
protected:
	CP_M_Persist						*fPersistObject;	/*! The CP_M_Persist object we hold. */

	CP_DataValueList					fDataValues;		/*! List of names/data value paris used to initialize the persist object */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!	\fn typedef CP_ObjectDescriptionArray
 *	A std::vector object that holds a list of CP_ObjectDescriptions used to initialize CP_M_Persist objects. 
 */
typedef std::vector<CP_ObjectDescription, std::allocator<CP_ObjectDescription> >								CP_ObjectDescription_Array;
typedef std::vector<CP_ObjectDescription, std::allocator<CP_ObjectDescription> >::iterator						CP_ObjectDescription_Iterator;
typedef std::vector<CP_ObjectDescription, std::allocator<CP_ObjectDescription> >::reverse_iterator				CP_ObjectDescription_ReverseIterator;

#if TARGET_CC_METROWERKS
#pragma mark === CP_ObjectDescriptions ==
#endif

/*!
*	@class	CP_ObjectDescriptions
*	@brief	A class to manage a list of CP_ObjectDescriptions that are used to create
*			objects in a CP_M_Persistant state
*/
class CP_EXPORT CP_ObjectDescriptions : CP_NoCopy
{
public:
											// Construction / destruction
										CP_ObjectDescriptions();
	virtual								~CP_ObjectDescriptions();
			
			UInt32						GetCount();
			void						Append( CP_ObjectDescription& inObjectDescription );
			CP_ObjectDescription		GetNthItem( UInt32 inIndex );
			
protected:

	// class data
protected:
	CP_ObjectDescription_Array			fObjectDescriptions;	/*! List of CP_ObjectDescription objects. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ObjectDescriptions

