//=========================================================================
//
//	FILE:				CP_XMLDocument.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle XML documents and the ability to both
//						parse into a tree structre and write back out.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 267 $
//						Latest Checkout:  $Date: 2005-08-31 19:33:20 -0700 (Wed, 31 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_XMLDocument
#define _H_CP_XMLDocument
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Data_ResourceTypes.h"
#include "CP_FileSystemSpec.h"
#include "CP_HandleBlock.h"
#include "CP_InStream.h"
#include "CP_M_XMLHandler.h"
#include "CP_NoCopy.h"
#include "CP_XMLWriter_Element.h"

	// expat
#include <expat.h>

	// Standard Library
#include <vector>
#include <stack>

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_M_XMLHandler;
class CP_XMLAttributes;
class CP_XMLDocNode;

	// typedefs
#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_XMLDocNode *, std::allocator<CP_XMLDocNode *> >					CP_DocNodes_Array;
	typedef std::vector<CP_XMLDocNode *, std::allocator<CP_XMLDocNode *> >::iterator		CP_DocNodes_Iterator;
#else
	typedef std::vector<CP_XMLDocNode *>													CP_DocNodes_Array;
	typedef std::vector<CP_XMLDocNode *>::iterator											CP_DocNodes_Iterator;
#endif

typedef enum CP_E_XMLDocNodeType
{
	CP_DocNodeType_ElementName,
	CP_DocNodeType_CharData
} CP_E_XMLDocNodeType;

#if TARGET_CC_METROWERKS
#pragma mark === CP_XMLDocument ===
#endif

/*!
*	@class		CP_XMLDocument
*	@brief		A class to handle XML documents and the ability to both parse into a tree
*				structre and write back out.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLDocument : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLDocument();
	//! Destructor.		
	virtual								~CP_XMLDocument();
	//@}

	//! @name Child Nodes
	//@{
										/*!
										*	Returns the outter most or root node of this XML document.
										*	
										*	@exception	none
										*	
										*	@return		CP_XMLDocNode *		The root node.
										*/
			CP_XMLDocNode				*GetRootNode()									{ return( fRootNode ); }

										/*!
										*	Sets the root node for this document.
										*
										*	@param		inRootNode			The root node for this documents.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetRootNode( CP_XMLDocNode *inRootNode )		{ fRootNode = inRootNode; }
	//@}

	//! @name Parsing
	//@{
			void						ParseXML( const CP_FileSystemSpec& inXMLFileSpec, bool inUseOSNative = true );
			void						ParseXML( const CP_HandleBlock& inXMLData );
			void						ParseXML( CP_InStream& inXMLStream );
	//@}

	//! @name Writing
	//@{
			void						WriteXML(
												const CP_FileSystemSpec& inXMLFileSpec,
												FourCharCode inCreator = 0,
												FourCharCode inType = kCP_Text_FileType );
			CP_HandleBlock 				*WriteXML();
	//@}

protected:

private:
	//! @name Writing
	//@{
			void						WriteNode(
												CP_XMLDocNode *inNode,
												CP_XMLWriter_Element *inParentElement );
			void						WriteChildNodes(
												CP_XMLDocNode *inNode,
												CP_XMLWriter_Element *inParentElement );
	//@}
			
	// class data members
protected:
	CP_XMLDocNode						*fRootNode;			/*! The root node for this XML document. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_XMLDocReader ===
#endif

/*!
*	@class		CP_XMLDocReader
*	@brief		A class to read an XML document and populate a passed in CP_XMLDocument object
*				from the XML.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLDocReader : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLDocReader(
												CP_InStream *inOutStream,
												CP_XMLDocument *inXMLDocument );
	//! Destructor.		
	virtual								~CP_XMLDocReader();
	//@}

	//! @name Parsing
	//@{
	virtual	void						Parse( bool inUseOSNative = true );

										/*!
										*	Returns the XML_Parser object used in parsing this document.
										*	
										*	@exception	none
										*	
										*	@return		XML_Parser		The XML parser used to parse this document.
										*/
			XML_Parser					GetParser()										{ return( fParser ); }
	//@}

protected:
	//! @name Error Handling
	//@{
	virtual	void						OnXMLParseError(
												SInt16 inLineNumber,
												SInt16 inXMLError );
	//@}

	//! @name XML Elements
	//@{
	virtual	void						OnStartElement(
												const XML_Char *inElement,
												CP_XMLAttributes& inAttributes );
	virtual	void						OnEndElement( const XML_Char *inElement );

	virtual	void						OnStartCData();
	virtual	void						OnEndCData();

	virtual	void						OnCharData(
												const XML_Char *s,
												SInt16 inLength );

	virtual	void						OnProcessingInstruction(
												const XML_Char *inTarget,
												const XML_Char *inData );
												
	virtual	void						OnComment( const XML_Char *inData );
	//@}

private:
	//! @name Parser Callbacks
	//@{
	static	void						StartElement_callback(
												void *inData,
												const XML_Char *inElement,
												const XML_Char **inAttributes );
	static	void						EndElement_callback(
												void *inData,
												const XML_Char *inElement );
												
	static	void						StartCData_callback( void *inData );
	static	void						EndCData_callback( void *inData );

	static	void						CharData_callback(
												void *inData,
												const XML_Char *s,
												int inLength );

	static	void						ProcessingInstruction_callback(
												void *inUserData,
												const XML_Char *inTarget,
						 						const XML_Char *inData );
	
	static	void						Comment_callback(
												void *inUserData,
												const XML_Char *inData );
	//@}
			
	// class data members
protected:
	CP_XMLDocument						*fXMLDocument;			/*! The CP_CP_XMLDocument that we are sending data to. */
	CP_InStream							*fInStream;				/*! The CP_InStream containing the XML data. */
	UInt32								fMaxBufferSize;			/*! Maximum size of the data we read each time. */
	XML_Parser							fParser;				/*! The XML parser object used to parse our data. */
	std::stack<CP_XMLDocNode *>			fNodes;					/*! Stack of nodes read in. */
	bool								fProcessingCharData;	/*! Flag to indicate if we are currently processing character data. */
	CP_String							fText;					/*! The accumulated character data for this node. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Child Nodes
	//@{
			void						AddChildNodes(
												CFXMLTreeRef inTreeRef,
												CP_XMLDocNode *inParentNode );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_XMLDocNode ===
#endif

/*!
*	@class		CP_XMLDocNode
*	@brief		A class to represent a single XML element in a CP_XMLDocument.
*
*	@ingroup	XML
*/
class CP_EXPORT CP_XMLDocNode
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLDocNode( const XML_Char *inNodeName );
	virtual								~CP_XMLDocNode();
	//@}
									
	//! @name Child Nodes
	//@{
			CP_XMLDocNode 				*AddChildNode( const XML_Char *inNodeName );
			bool						HasChildren();
			UInt32						GetChildNodeCount();
			CP_XMLDocNode 				*GetNthChildNode( UInt32 inIndex );

										/*!
										*	Returns a list of the child nodes.
										*	
										*	@exception	none
										*	
										*	@return		CP_DocNodesArray		A list of the child nodes.
										*/
			CP_DocNodes_Array			*GetChildNodeList()								{ return( fChildNodes ); }
	//@}
										
	//! @name Getters
	//@{
										/*!
										*	Returns the type of node.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_XMLDocNodeType	The type of node.
										*/
			CP_E_XMLDocNodeType			GetNodeType()									{ return( fNodeType ); }

										/*!
										*	Returns the name of this node.
										*	
										*	@exception	none
										*	
										*	@return		CP_String			The name of the node.
										*/
			CP_String					GetNodeName()									{ return( fNodeName ); }

										/*!
										*	Returns true if this node contains character data.
										*	
										*	@exception	none
										*	
										*	@return		bool				True if this node contains character data.		
										*/
			bool						HasCharData()									{ return( fCharData.length() > 0 ); }

										/*!
										*	Returns the character data for the node.
										*	
										*	@exception	none
										*	
										*	@return		CP_String			The string contains the character data.		
										*/
			CP_String					GetCharData()									{ return( fCharData ); }

										/*!
										*	Returns a CP_XMLAttributes object for the attributes, if any, for
										*	this node.
										*	
										*	@exception	none
										*	
										*	@return		CP_XMLAttributes	The attributes, if any, for this node.
										*/
			CP_XMLAttributes			GetAttributes()									{ return( fAttributes ); }
	//@}
			
	//! @name Setters
	//@{
										/*!
										*	Sets the type of node this is.
										*
										*	@param		inType				The CP_E_XMLDocNodeType of this node.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetNodeType( CP_E_XMLDocNodeType inType )		{ fNodeType = inType; }

										/*!
										*	Sets the character data for the node from the passed in CP_String.
										*
										*	@param		inCharData			A CP_String containing the character data
										*									for this node.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetCharData( const CP_String& inCharData )		{ fCharData = inCharData; }

										/*!
										*	Sets the attributes for this node.
										*
										*	@param		inAttributes		A CP_XMLAttributes object containing the attributes
										*									for this node.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetAttributes( CP_XMLAttributes& inAttributes )	{ fAttributes = inAttributes; }
	//@}
			
	// class data members
protected:
		// node identification
	CP_E_XMLDocNodeType					fNodeType;		/*! Type of node. */
	CP_String							fNodeName;		/*! The name of the node as it appears in the XML file. */
	
		// node data
	CP_String							fCharData;		/*! Character data associated with this node. */
	CP_XMLAttributes					fAttributes;	/*! Any attributes for this node. */
	
		// child nodes
	CP_DocNodes_Array					*fChildNodes;	/*! List of any child nodes. */
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_XMLDocNode( CFXMLTreeRef inTreeRef );
	//@}

	//! @name Child Nodes
	//@{
			CP_XMLDocNode 				*AddChildNode( CFXMLTreeRef inTreeRef );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_XMLDocument


